<?php
/**
 *
 * @category        snippets
 * @package         minified
 * @author          Ruud Eisinga - Dev4me
 * @link			http://www.dev4me.nl/
 * @license         http://www.gnu.org/licenses/gpl.html
 * @platform        WebsiteBaker 2.8.x
 * @requirements    PHP 5.6 and higher
 * @version         1.1
 * @lastmodified    March 16, 2018 
 *
 */


if(defined('WB_PATH') == false) { exit("Cannot access this file directly"); }

$path = dirname(__FILE__).'/lib';
require_once $path . '/minify/src/Minify.php';
require_once $path . '/minify/src/CSS.php';
require_once $path . '/minify/src/JS.php';
require_once $path . '/minify/src/Exception.php';
require_once $path . '/minify/src/Exceptions/BasicException.php';
require_once $path . '/minify/src/Exceptions/FileImportException.php';
require_once $path . '/minify/src/Exceptions/IOException.php';
require_once $path . '/path-converter/src/ConverterInterface.php';
require_once $path . '/path-converter/src/Converter.php';

use MatthiasMullie\Minify;

global $css, $js;

function minify_css($add_frontend = true) {
	global $css;
	$key = _get_minify_key('css');
	if ($add_frontend) _get_frontend('css');
	if(_need_minify('css')) {
		$key = _get_minify_key('css',true);
		$css_minifier = new Minify\CSS('');
		foreach ( $css as $cssfile ) {
			$css_minifier->add(_minify_filename($cssfile));
		}
		$output = _minify_filename('minified-'.$key.'.css');
		$css_minifier->minify($output);
	}
	echo '<link href="'.TEMPLATE_DIR.'/minified-'.$key.'.css" rel="stylesheet" />'.PHP_EOL;
} 

function minify_js($add_frontend = true) {
	global $js;
	$key = _get_minify_key('js');
	if ($add_frontend) _get_frontend('js');
	if(_need_minify('js')) {
		$key = _get_minify_key('js',true);
		$js_minifier = new Minify\JS('');
		foreach ( $js as $jsfile ) {
			$js_minifier->add(_minify_filename($jsfile));
		}
		$output = _minify_filename('minified-'.$key.'.js');
		$js_minifier->minify($output);
	}
	echo '<script src="'. TEMPLATE_DIR .'/minified-'.$key.'.js"></script>'.PHP_EOL;
} 

function _minify_filename($relative_file, $get_content = true) {

	if(substr($relative_file,0,2) == "//") $relative_file = 'https:'.$relative_file;
	if(substr($relative_file,0,5) == "http:" || substr($relative_file,0,6) == "https:") {
		if($get_content) return file_get_contents($relative_file);
		return null;
	}	
	$filename = null;
	if(preg_match('/^[A-Z]:/i', $relative_file)) {  	// detect windows absolute path
		$filename = $relative_file;
	} elseif(substr($relative_file,0,1) != "/") { 		// relative to template dir
		$template_path = str_ireplace(WB_URL,WB_PATH,TEMPLATE_DIR);
		$filename = $template_path.'/'.$relative_file;
	} elseif (file_exists( $relative_file )) {  		// direct file
		$filename = $relative_file;
	} elseif (file_exists( WB_PATH.$relative_file )) {  // relative to WB root dir
		$filename = WB_PATH.$relative_file;
	}
	//if(!$filename) _do_log($relative_file.' does not exist');
	return $filename;
}

function _need_minify($type = 'css') {
	global $css, $js;
	$key = _get_minify_key($type);
	if(!isset($_SESSION['USER_ID'])) return false;  //rebuild only when logged in user
	if(isset($_SESSION['USER_ID'])) {  //Detect CTRL-F5 or ?force parameter only when logged in user
		if(isset($_GET['force'])) return true;
		if(isset($_SERVER['HTTP_CACHE_CONTROL']) && $_SERVER['HTTP_CACHE_CONTROL'] === 'no-cache') return true;
	}
	if($type == 'css') {
		$ofile = filemtime(_minify_filename('minified-'.$key.'.css',false));
		if(!$ofile) return true;
		//if(isset($_SESSION['mini-css']) && $_SESSION['mini-css'] == $css) return false;
		//$_SESSION['mini-css'] = $css;
		foreach ( $css as $cssfile ) {
			if($thefile = _minify_filename($cssfile,false)) {
				if(filemtime($thefile) > $ofile) return true;
			}
		}
	} elseif($type == 'js') {
		$ofile = filemtime(_minify_filename('minified-'.$key.'.js',false));
		if(!$ofile) return true;
		//if(isset($_SESSION['mini-js']) && $_SESSION['mini-js'] == $js) return false;
		//$_SESSION['mini-js'] = $js;
		foreach ( $js as $jsfile ) {
			if($thefile = _minify_filename($jsfile,false)) {
				if(filemtime($thefile) > $ofile) return true;
			}
		}

	}
	return false;
	
}
function _get_frontend($type= 'css') {
	global $database,$css,$js;
	$query = "SELECT DISTINCT `directory` FROM ".TABLE_PREFIX."addons where function='snippet'";
	$res = $database->query($query);
	if($res && $res->numRows() > 0) {
		while ($row = $res->fetchRow()) {
			$fpath = WB_PATH.'/modules/'.$row['directory'];
			if($type=='css' && file_exists($fpath.'/frontend.css')) $css[] = $fpath.'/frontend.css';
			if($type=='js' && file_exists($fpath.'/frontend.js')) $js[] = $fpath.'/frontend.js';
		}
	}	
	$query = "SELECT DISTINCT `directory` FROM ".TABLE_PREFIX."addons a JOIN
			 ".TABLE_PREFIX."sections s ON s.module = a.directory";
	$res = $database->query($query);
	if($res && $res->numRows() > 0) {
		while ($row = $res->fetchRow()) {
			$fpath = WB_PATH.'/modules/'.$row['directory'];
			if($type=='css' && file_exists($fpath.'/frontend.css')) $css[] = $fpath.'/frontend.css';
			if($type=='js' && file_exists($fpath.'/frontend.js')) $js[] = $fpath.'/frontend.js';
		}
	}
	$done = 1;
}

function _do_log($message) {
    $sLogfile = ini_get ('error_log');
    if (!is_writeable($sLogfile)){return false;}
	$sMsg = date('c').' '.'[MINIFY] "'.$message.'"'.PHP_EOL;
    file_put_contents($sLogfile, $sMsg, FILE_APPEND);
}

function _get_minify_key($type = 'css', $renew = false) {
	global $database;
	$key = $database->get_one("SELECT `value` FROM `".TABLE_PREFIX."settings` WHERE `name`='minify_key_$type' LIMIT 1");
	if(!$key || $renew) {
		$path = str_ireplace(WB_URL,WB_PATH,TEMPLATE_DIR);
		if(file_exists($path.'/minified-'.$key.'.css')) unlink($path.'/minified-'.$key.'.css');
		if(file_exists($path.'/minified-'.$key.'.js')) unlink($path.'/minified-'.$key.'.js');
		$key = md5(microtime());
		$database->query("INSERT INTO `".TABLE_PREFIX."settings` (`name`, `value`) VALUES ('minify_key_$type', '$key') ON DUPLICATE KEY UPDATE `value`='$key'");
	}
	return $key;
}